<?php

abstract class AMQPBase {

    // Ports
    const PORT = 5672;
    const SSL_PORT = 5671;

    // Rabbit MQ Virtual Host
    const VHOST = '/';
    const LOGIN_METHOD = 'AMQPLAIN';
    const LOCALE = 'en_US';

    // Base directory for SSL certificates
    const CERTS_BASE_DIR = '/usr/local/mgportal/certs/';

    public function __construct($host = null, $user = null, $asset = null) {
        $this->channel = null;
        $this->connection = null;
        $this->exchange = null;
        $this->queue = null;
        $this->port = AMQP::PORT;       // default to standard RabbitMQ port
        $this->vHost = AMQP::VHOST;     // default to standard vHost

        $this->useSSL = false;          // default to non-SSL connection
        $this->heartbeat = 0;           // default to no hearteat
        $this->timeout = null;          // default to no timeout

        $this->host = null;
        $this->user = null;
        $this->assetTag = $asset;
    }

    protected function setParameters($host, $user) {
        $this->host = $host;
        $this->user = $user;
    }

    public function setSSLOptions($CertsBaseDir) { //CADir, $clientDir){
        $CertsBaseDir .= $this->host . '/';
        $CADir = $CertsBaseDir . 'testca/';
        $clientDir = $CertsBaseDir . 'client/';
        $CACert= $CADir . "cacert.pem";
        $cert = $clientDir . "cert.pem";
        $privKey = $clientDir . "key.pem";

        // If the directories do not exist, create them.
        if (!is_dir($CADir)) {
            mkdir($CADir, 0775, true);
            $this->dir_modify($CADir, $this->host, 'mgportal', 'mgportal');
        }
        if (!is_dir($clientDir)) {
            mkdir($clientDir, 0775, true);
            $this->dir_modify($clientDir, $this->host, 'mgportal', 'mgportal');
        }
        // If the SSL files do not exist, get them.
        if (!file_exists($CACert)) {
            $portalREST = new REST(null, false);
            $portalREST->setAuthFromAsset($this->assetTag, true);
            $urlType = "GET";
            $command = "rabbitmq.php";
            $options = "?client_keys=1";
            $result = $portalREST->run($command, $urlType, $this->host, null, $options);
            if ($portalREST->isError($result)) {
                global $Log;
                $Log->logError($result . " (Host: " . $this->host . ")");
            } else {
                if (is_array($result)) {
                    if (isset($result['ca'])) {
                        file_put_contents($CACert, $result['ca']);
                    }
                    if (isset($result['cert'])) {
                        file_put_contents($cert, $result['cert']);
                    }
                    if (isset($result['private'])) {
                        file_put_contents($privKey, $result['private']);
                    }
                }
            }
        }
        $ssl_options = array(
            'cafile' => $CACert,
            'verify_peer' => true,
            'allow_self_signed' => true,
            'local_cert' => $cert,
            'local_pk' => $privKey
        );
        return $ssl_options;
    }

    private function dir_modify($dir, $startDir, $owner, $group) {
        $aPath = explode('/', $dir);
        $pathName = '/';

        $startChange = false;
        foreach ($aPath as $subDir) {
            if ($subDir == $startDir) {
                $startChange = true;
            }
            $pathName .= $subDir . '/';
            if ($startChange) {
                chown($pathName, $owner);
                chgrp($pathName, $group);
            }
        }
    }

    public function setPort($port) {
        $this->port = $port;
    }

    public function setQueue($queue) {
        $this->queue = $queue;
    }

    public function getQueue() {
        return $this->queue;
    }

    public function getChannel() {
        return $this->channel;
    }

    public function getExchange() {
        return $this->exchange;
    }

    public function getConnection() {
        return $this->connection;
    }

    public function setTimeout($timeout) {
        $this->timeout = $timeout;
    }

    abstract public function setWaitTimeout($timeout);

    public function setHeartbeat($heartbeat) {
        $this->heartbeat = $heartbeat;
    }
}

?>