<?php

class AMQP extends AMQPBase {

    public function __construct($host = null, $user = null, $asset = null) {
        parent::__construct($host, $user);
        $this->connection = new AMQPConnection();
        if ($host != null) {
            $this->setParameters($host, $user);
        }
    }

    public function setParameters($host, $user) {
        parent::setParameters($host, $user);
        if ($this->host != null) {
            $this->connection->setHost($this->host);
        }
    }

    public function connect($useSSL = false) {
        $this->connection->setLogin($this->user['name']);
        $this->connection->setPassword($this->user['password']);
        $this->connection->setPort($this->port);
        if ($useSSL) {
            // If the port was left as the default, use the default SSL port.
            if ($this->port == AMQPBase::PORT) {
                $this->port = AMQPBase::SSL_PORT;
            }
            $ssl_options = $this->setSSLOptions(AMQPBase::CERTS_BASE_DIR);
            $this->connection->setCACert($ssl_options['cafile']);
            $this->connection->setCert($ssl_options['local_cert']);
            $this->connection->setKey($ssl_options['local_pk']);
            $this->connection->setVerify(true);
        }
        $this->connection->connect();
    }

    public function createChannel() {
        $this->channel = new AMQPChannel($this->connection);
        return $this->getChannel();
    }

    public function createExchange() {
        $this->exchange = new AMQPExchange($this->channel);
        //$this->exchange->declare('cloud-default');
        return $this->getExchange();
    }

    public function createQueue($routing_key) {
        $this->queue = new AMQPQueue($this->channel);
        $this->queue->setName($routing_key);
        $this->queue->setFlags(AMQP_NOPARAM);
        $this->queue->declareQueue();
        return $this->getQueue();
    }

    public function disconnect() {
        $this->connection->disconnect();
    }

    public function setTimeout($timeout) {
        // do nothing
    }

    public function reconnect() {
        $this->connection->reconnect();
    }

    public function setWaitTimeout($timeout) {
        // set wait timeout for incoming (read) data.
        $this->connection->setReadTimeout($timeout);
    }
}

?>
