<?php
/**
 *  Database functions
 */

class Database {
    const DB_CONNECTION_STRING = "host=localhost port=5432 dbname=mgportal user=postgres connect_timeout=30";
    const CONNECT_ERROR = "Unable to connect to database.";
    
    public function __construct() {
        $this->connection = null;
    }

    public function __destruct() {
        $this->closeConnection();
    }
    
    public function connect() {
        $this->connection = pg_connect(Database::DB_CONNECTION_STRING);
        return $this->getConnection();
    }
    
    public function getConnection() {
        return $this->connection;
    }
    
    public function closeConnection() {
        if ($this->connection !== null && $this->connection !== false) {
            pg_close($this->connection);
            $this->connection = null;
        }
    }

    public static function db_query($connection, $Log, $query) {
        $result = false;
        if ($connection !== null) {
            if ($Log !== null) {
                $Log->logDebug("pg_query: " . $query);
            }
            $result = pg_query($connection, $query);
            if ($Log !== null) {
                $Log->writeVariable("result: " . $result, 1, false, Logger::DEBUG);
            }
        } else {
            if ($Log !== null) {
                $Log->logDebug("pg_query: connection is null");
            }
        }
        return $result;
    }

    public static function db_fetch_row($Log, $result) {
        $row = pg_fetch_row($result);
        if ($Log !== null) {
            $Log->logDebug("pg_fetch_row");
            $Log->writeVariable($row, 1, false, Logger::DATABASE);
        }
        return $row;
    }

    public static function db_num_rows($Log, $result) {
        $rowCount = pg_num_rows($result);
        if ($Log !== null) {
            $Log->logDebug("pg_num_rows: " . $rowCount);
        }
        return $rowCount;
    }

    public static function db_affected_rows($Log, $result) {
        $rowCount = pg_affected_rows($result);
        if ($Log !== null) {
            $Log->logDebug("pg_affected_rows: " . $rowCount);
        }
        return $rowCount;
    }

    public static function db_escape_string($data) {
        if (function_exists("pg_escape_literal")) {
            // PHP 5.4+
            $escaped = pg_escape_literal($data);
        } else {
            // PHP 5.3 or lower
            $escaped = "'" . pg_escape_string($data) . "'";
        }
        return $escaped;
    }
};

?>