#!/bin/sh

AMQPLIB_VERSION="2.6"
MGPORTAL_HOME=/home/mgportal
MGPORTAL_BIN=/usr/local/mgportal/bin
COMPOSER=$MGPORTAL_BIN/composer.phar

# see if composer is already installed
if [ ! -f $COMPOSER ]
then
    # check dependencies
    echo "Checking dependencies...."
    rpm -qa | grep php-bcmath | grep -v grep > /dev/null
    if [ $? != 0 ]
    then
        echo "Missing php dependencies php-bcmath and/or php-bmstring.  Please ensure both are installed and try again.";
        exit -1
    else
        rpm -qa | grep php-mbstring | grep -v grep > /dev/null
        if [ $? != 0 ]
        then
            echo "Missing php dependency php-bmstring.  Please install the package and try again.";
            exit -1
        fi
    fi

    echo "Dependencies okay.  Downloading composer utility to install AMQPlib....."

    # download composer.phar
    cd $MGPORTAL_BIN
    $MGPORTAL_BIN/install_composer.sh
    if [ $? != 0 ]
    then
        echo "Error downloading composer.  Please ensure appliance can connect to the network and retry.";
        exit -1
    fi

    echo "Composer download completed successfully.  Installing AMQPlib....."
else
    echo "Composer is already downloaded.... Installing AMQPlib....."
fi

# install with the json config.
cd $MGPORTAL_HOME
cat <<EOF > $MGPORTAL_HOME/composer.json
{
  "require": {
      "php-amqplib/php-amqplib": "$AMQPLIB_VERSION.*"
  }
}
EOF

# set file mode, cannot run composer as root
chown mgportal:mgportal $MGPORTAL_HOME/composer.json
su mgportal -c "$MGPORTAL_BIN/composer.phar install"
if [ $? = 0 ]
then
    echo "AMQPlib installation complete."
else
    echo "AMQPlib installation failed.  You will need to use php-pecl-amqp instead."
fi