#!/usr/bin/python

#
# Merge JSON configuration files
#
# This program merges two JSON configuration files, where the first is preserved and the second is merged into the first.
# The output is saved to another json file.
#

import json
import sys
from pprint import pprint

# Number of arguments (including command)
argc = len(sys.argv)

# get arguments
argv = sys.argv

if argc != 4:
	print
	print "Error: ", argv[0], ": invalid usage"
	print "Usage: json_merge_conf <original file> <new file> <merged file>"
	print
	exit(-1)

#
# The first argument is the original configuration file
# The second argument is the new configuration file.
# The third argument is the merged configuration file.
#
original_file = argv[1]
new_file = argv[2]
merged_file = argv[3]


#
# Load json file, returning the jsonData objects
#
def loadFile(fileName):
	try:
		fp = open(fileName, "r")
		jsonData = json.load(fp)
		fp.close()
	except IOError:
		print "Error opening config file", fileName, "for reading"
		sys.exit(-1)
	except ValueError as err:
		print "Error loading json from config file", fileName, err
		sys.exit(-1)
	except:
		print "Unexpected error opening file", fileName, sys.exc_info()[0]
	return jsonData

#
# Dump json objects to output file, pretty printed
#
def writeFile(fileName, jsonData):
	try:
		fp = open(fileName, "w")
		fp.write(json.dumps(jsonData, indent=4, sort_keys=True))
		fp.close()
	except IOError:
		print "Error opening config file", fileName, "for writing"
		sys.exit(-1)
	except:
		print "Unexpected error writing file", fileName, sys.exc_info()[0]

#
# If the list contains the property and value, return true, else false.
#
def listContainsProperty(list, property, value):
	found = False
	for o in list:
		if o[property] == value:
			found = True
			break;
	return found


#
# Merge two lists by iterating over them, key'ing off of one significant property
# Returns merged list
def mergeLists(list_save, list_new, property, area):
	merged = []
	for i, listSaveObj in enumerate(list_save):
		found = False
		for j, listNewObj in enumerate(list_new):
			if listSaveObj[property] == listNewObj[property]:
				print "merging obj", listSaveObj, "and", listNewObj
				obj = mergeObjects(listSaveObj, listNewObj, area)
				merged.append(obj)
				found = True
				break
		if not found:
			merged.append(listSaveObj)

	for j, listNewObj in enumerate(list_new):
		if not listContainsProperty(merged, property, listNewObj[property]):
			merged.append(listNewObj)

	print "merged list is", merged
	return merged


#
# Merge two objects by iterating over properties, returned merged object
#
def mergeObjects(obj_save, obj_new, area):
	merged = {}
	for key, val in obj_save.iteritems():
		if (type(val) is list):
	        	if key in obj_new:
	            		# Merge using 'host' property of the objects as unique key
			    	val = mergeLists(obj_save[key], obj_new[key], 'host', area)
            		else:
                		print "key", key, "does not exist in second file"

		print "saving", area, "key/value", key, val
		merged[key] = val

	for key, val in obj_new.iteritems():
		if key not in merged:
			merged[key] = val
			print "add new", area, "key/value", key, val

	return merged


#
# Merge the new and saved data objects base on the area property
# Return merged data, empty if no merge possible
#
def mergeData(data_new, data_save, area):
	merged_data = {}
	if area in data_new and area in data_save:
		area_new = data_new[area]
		area_save = data_save[area]
		merged_data = mergeObjects(area_new, area_save, area)
	elif area in data_new:
		merged_data = data_new[area]
	elif area in data_save:
		merged_data = data_save[area]
	else:
		print "No", area, "configuration information to merge, skipping..."
	return merged_data

# Main program

data_new = loadFile(original_file)
data_save = loadFile(new_file)

# Create JSON object file data
file_data = {}

# Merge client data
client_data = mergeData(data_new, data_save, 'client')
if client_data:
	file_data['client'] = client_data

# Merge server data
server_data = mergeData(data_new, data_save, 'server')
if server_data:
	file_data['server'] = server_data

# saved merged data to new file
writeFile(merged_file, file_data)
