<?php

class vendor {
    public function __construct($REST, $Config) {
        $this->REST = $REST;
        $this->Config = $Config;
    }
    
    public function run_command($command, $days, $host = null) {
        $message = "";
        switch ($command) {
            case Configuration::CMD_PROTECT_INFO:
                $message = $this->get_protection_information();
                break;
            case Configuration::CMD_PROTECTED_MONTHLY:
                $message = $this->get_month_end();
                break;
            case Configuration::CMD_DEVICE_CHECK:
                $message = $this->get_version();
                break;
            case Configuration::CMD_GET_JOBS:
                $message = $this->get_job_list();
                break;
            case Configuration::CMD_GET_ALERTS:
                $message = $this->get_alert_list();
                break;
            case Configuration::CMD_GET_BACKUPS:
                $message = $this->get_backup_list($days);
                $message['data_days'] = $days;
                break;
            case Configuration::CMD_GET_HOT_COPIES:
                $message = $this->get_replication_list($days);
                $copies = array();
                if (isset($message['source_data'])) {
                    $copies = $message['source_data'];
                }
                $message = array('data' => $copies);
                $message['data_days'] = $days;
                break;
            case Configuration::CMD_ADD_KEY:
                $message = $this->get_source_key($days);
                break;
            case Configuration::CMD_GET_KEY:
            case Configuration::CMD_GET_CONNECTION_STATUS:
                // do nothing
                break;
            case Configuration::CMD_GET_ALL:
                $message = $this->send_data_to_cloud($host, $days);
                break;
            case Configuration::CMD_OPEN_TUNNEL:
                $message = $this->build_open_tunnel($host);
                break;
            case Configuration::CMD_CLOSE_TUNNEL:
                $message = $this->close_tunnel($host);
                break;
            case Configuration::CMD_CHECK_TUNNEL:
                $message = $this->check_tunnel($host);
                break;
        }

        // Add the standard header to the message (command name, timestamp, etc.)
        $this->Config->add_message_header($message, $command);
        
        return $message;
    }

    private function get_backup_list($days){
        $api = 'reports/backup_history';
        $this->REST->setOptions($this->getDaysURI($days));
        return $this->REST->run($api);
    }

    private function get_replication_list($days){
        $api = 'reports/replication_history';
        $this->REST->setOptions($this->getDaysURI($days));
        return $this->REST->run($api);
    }

    private function get_source_key($filename) {
        $contents = file_get_contents($filename);
        if ($contents !== false) {
            $message = array('data' => $contents);
        } else {
            $message = false;
        }
        return $message;
    }

    private function getDaysURI($days) {
        $uri = "";
        if ($days != -1) {
            $now = time();
            $end = date("m/d/Y", $now);
            $start = date("m/d/Y", $now - (($days - 1) * 86400));
            $uri = '&start_date=' . $start . '&end_date=' . $end;
        }
        return $uri;
    }

    private function get_alert_list(){
        return $this->REST->run('alerts');
    }

    private function get_version() {
        $message = array();
        $applianceInfo = $this->REST->run("systems");
        if (!$this->REST->isError($applianceInfo)) {
            $applianceInfo = $applianceInfo['appliance'][0];
            $message['version'] = $applianceInfo['version'];
        }
        return $message;
    }

    private function get_job_list() {
        return $this->REST->run("joborders");
    }

    private function get_protection_information() {
        $system_capacity = $this->REST->run('reports/system_capacity');
        if ($this->REST->isError($system_capacity)) {
            $message = false;
            return $message;
        }
        $message = $system_capacity;
        if (isset($message['data'])) {
            $data = $message['data'];
            // $data is array of elements, get first.
            if (is_array($data)) {
                $data = $data[0];
            }
            $message['server_count'] = $message['vm_count'] = $message['app_count'] = 0;
            // Loop through all items, get the count of protected items.
            foreach ($data as $key => $value) {
                if (is_array($value)) {
                    if ($key == 'clients') {
                        $message['server_count'] += $this->get_protected_count($value);
                    } else if ($key == 'vms') {
                        $message['vm_count'] += $this->get_protected_count($value);
                    } else {
                        $message['app_count'] += $this->get_protected_count($value);
                    }
                }
            }
        }

        $replicating_assets = $this->REST->run('replication/assets');
        $replicating_count = 0;
        $replicating_vms = 0;
        if (isset($replicating_assets['replicating'])) {
            $assets = $replicating_assets['replicating'];
            foreach ($assets as $asset) {
                $replicating_count++;
                if ($this->is_vm($asset)) {
                    $replicating_vms++;
                }
            }
        }
        $message["all_replicating"] = $replicating_count;
        $message["vm_replicating"] = $replicating_vms;

        if ($replicating_count > 0) {
            $message['replication_used'] = $this->get_replicated_capacity();
        } else {
            $message['replication_used'] = 0;
        }

        return $message;
    }

    private function get_replicated_capacity() {
        $gb_size = 0;
        $replicated_capacity = $this->REST->run('reports/replication/capacity');
        if (!$this->REST->isError($replicated_capacity)) {
            $result = $replicated_capacity;
            if (isset($result['data'])) {
                $assetList = $result['data'];
                $mb_size = 0;
                foreach ($assetList as $asset) {
                    $mb_size += $asset['size'];
                }
            }
            $gb_size = round(((float)$mb_size / 1024.0), 2);
        }
        return $gb_size;
    }
    
    /*
     * Given an array of items, return the count of ones with a last_full backup.
     */
    private function get_protected_count($items) {
        $count = 0;
        foreach ($items as $asset) {
            if (isset($asset['last_full']) && $asset['last_full'] > 0) {
                $count++;
            }
        }
        return $count;
    }

    private function get_month_end() {
        $message = array();

        // Original method - Query given to MSPs.
        $conn = pg_connect("port=5432 dbname=bpdb user=postgres connect_timeout=30");
        if ($conn !== false) {
            $query = "select count(distinct s.instance_id) as protected_assets from bp.successful_backups s inner join bp.nodes n ";
            $query .= "using (node_no) inner join bp.application_instances ai using (instance_id) inner join bp.application_lookup al ";
            $query .= "using (app_id) where al.app_id in (50, 51, 52, 40, 1) and (now()::date - 30) < (to_timestamp(s.start_time)::date);";
            $result = pg_query($conn, $query);
            if ($result !== false) {
                $row = pg_fetch_assoc($result);
                $message = $row;
            }
            pg_close($conn);
        }

        // New method - BPL API
        // Past 30 days
        //$now = time();
        //$lastMonth = $now - (30 * 86400);
        // Use strtotime to go back to beginning of day 30 days ago.
        $lastMonthDate = date("Y-m-d", strtotime("-30 days"));
        $lastMonth = strtotime($lastMonthDate);

        // Only file-level, VMware, and Hyper-V backups
        // app ID 1, 40, 51, 52
        $backupTypes = array ("master", "incremental", "differential", "selective", "baremetal",
                                "vmware full", "vmware differential", "vmware incremental",
                                "hyperv full", "hyperv incremental");
        global $Log;
        $Log->logInfo("Get backups since " . $lastMonthDate . " timestamp(" . $lastMonth . ")");
        bp_bypass_cookie(3);
        $backups = bp_get_backup_status(array('start_time' => $lastMonth,
                                              'type' => $backupTypes));
        $unique_instances = array();
        $unique_clients = array();
        if ($backups !== false) {
            foreach ($backups as $backup) {
                // Backups must be successful (completed, not failed)
                // Count number of uniques.  For clients, we get client_id, otherwise we get instance_id.
                if (isset($backup['status']) && $backup['status'] != 2) {
                    if (isset($backup['instance_id'])) {
                        if (!in_array($backup['instance_id'], $unique_instances)) {
                            $unique_instances[] = $backup['instance_id'];
                        }
                    } else if (isset($backup['client_id'])) {
                        if (!in_array($backup['client_id'], $unique_clients)) {
                            $unique_clients[] = $backup['client_id'];
                        }                       
                    }
                }
            }
        } else {
            $Log->logError("Error" . bp_error());
        }
        $uniques = count($unique_instances) + count($unique_clients);
        
        if (is_array($message)) {
            $message['protected_assets_api'] = $uniques;
        }
        return $message;
    }
    
    public function update_message(&$message, $license) {
        $message["asset_tag"] = $license['asset_tag'];
        $message["license_class"] = $license['class'];
        $message["license_mkt_name"] = $license["mkt_name"];
        $message["license_feature"] = $license['feature_string'];
        $message["install_date"] = $license["install_date"];
        if (isset($license['hostname'])) {
            $message['hostname'] = $license['hostname'];
        } else {
            $message['hostname'] = 'unknown';
        }
    }

    public function is_vm($asset) {
        $is_vm = false;

        $app_id = $asset['app_id'];

        switch ($app_id) {
            case 40:	// VMware
            case 50:	// Hyper-V
            case 51:
            case 52:
            case 140:	// Xen
                $is_vm = true;
                break;
        }

        return $is_vm;
    }

    private function build_open_tunnel($host) {
        // See if keys exist on client
        // Re-generate keys
        // Toggle Tunnel
        global $Log;
        //$password = $this->getPortalPassword();
        //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh testkeys " . $host;
        $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh testkeys " . $host;
        $output = system($cmd, $returnValue);
        $Log->logNotice("return of testkey is " . $returnValue . " and output is " . $output);
        if ($returnValue != 0) {
            //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh managerkey " . $host;
            $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh managerkey " . $host;
            $output = system($cmd, $returnValue);
            $Log->logNotice("return of get manager key is " . $returnValue . " and output is " . $output);
            if ($returnValue == 0) {
                //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh hostkey";
                $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh hostkey";
                $output = system($cmd, $returnValue);
                $Log->logNotice("return of generate host key is " . $returnValue . " and output is " . $output);
            }
        }
        //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/toggle_tunnel.sh " . $host;
        $cmd = "sudo -u mgportal /usr/local/mgportal/bin/toggle_tunnel.sh " . $host;
        $tunnelOutput = system($cmd, $returnValue);
        $Log->logNotice("return of toggle tunnel is " . $returnValue . " and output is " . $tunnelOutput);

        $result = array('meta_command' => 'tunnel', 'output' => $tunnelOutput, 'host' => $host);
        return $result;
    }

    private function close_tunnel($host) {
        // Toggle Tunnel
        global $Log;
        //$password = $this->getPortalPassword();
        //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/toggle_tunnel.sh " . $host;
        $cmd = "sudo -u mgportal /usr/local/mgportal/bin/toggle_tunnel.sh " . $host;
        $tunnelOutput = system($cmd, $returnValue);
        $Log->logNotice("return of toggle tunnel is " . $returnValue . " and output is " . $tunnelOutput);

        $result = array('meta_command' => 'tunnel', 'output' => $tunnelOutput);
        return $result;
    }

    private function check_tunnel($host) {
        // Toggle Tunnel
        global $Log;
        //$password = $this->getPortalPassword();
        //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh test " . $host;
        $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh test " . $host;
        $tunnelOutput = system($cmd, $returnValue);
        $Log->logNotice("return of test tunnel is " . $returnValue . " and output is " . $tunnelOutput);

        $result = array('meta_command' => 'tunnel', 'output' => $tunnelOutput, 'host' => $host);
        return $result;
    }

    private function send_data_to_cloud($host, $nDays) {
        // call to_cloud on demand.
        global $Log;
        //$password = $this->getPortalPassword();
        $days = isset($nDays) ? $nDays : $this->Config->getDays();
        //$cmd = "sudo -u mgportal -S /usr/local/mgportal/bin/to_cloud.php -c all -h " . $host . " -d " . $days;
        $cmd = "sudo -u mgportal /usr/local/mgportal/bin/to_cloud.php -c all -h " . $host . " -d " . $days;
        $Log->logNotice("cmd: " . $cmd);
        //$cmd = "echo " . $password . " | ". $cmd;
        exec($cmd, $arrayOutput, $returnValue);
        if ($returnValue != 0) {
            foreach ($arrayOutput as $line) {
                if (strpos($line, REST::ERROR_PREFIX) === 0) {
                    $to_cloud_output = $line;
                    break;
                }
            }
            if (!isset($to_cloud_output)) {
                $to_cloud_output = "Error: failure sending data to host " . $host . ".  Please check logs for more details.";
            }
        } else {
            $to_cloud_output = implode($arrayOutput);
        }
        $Log->logNotice("return of send info to cloud is " . $returnValue . " and output is " . $to_cloud_output);

        $result = array('meta_command' => 'to_cloud', 'success' => $returnValue === 0, 'output' => $to_cloud_output, 'host' => $host);
        return $result;
    }

    /*
     * Returns the portal password on the "client" side.
     */
    /*public function getPortalPassword() {
        $password = "dummy";
        $data = file_get_contents('/home/mgportal/.xgpw');
        if ($data !== false) {
            $decoded = base64_decode($data);
            // should be string <pw>.<timestamp>
            $a = explode('.', $decoded);
            $password = $a[0];
        }
        return $password;
    }*/

}

?>
